@@ -55,6 +55,7 @@ if MYPY_RUNNING:
     TFinder = TypeVar("TFinder")
     TResolver = TypeVar("TResolver")
     TReqTracker = TypeVar("TReqTracker")
+    TBuildTracker = TypeVar("TBuildTracker")
     TReqSet = TypeVar("TReqSet")
     TLink = TypeVar("TLink")
     TSession = TypeVar("TSession", bound=Session)
@@ -441,7 +442,7 @@ def get_session(
 def populate_options(
     install_command=None,  # type: TCommandInstance
     options=None,  # type: Optional[Values]
-    **kwargs  # type: Any
+    **kwargs,  # type: Any
 ):
     # (...) -> Tuple[Dict[str, Any], Values]
     results = {}
@@ -708,6 +709,7 @@ def shim_unpack(
     only_download=None,  # type: Optional[bool]
     downloader_provider=None,  # type: Optional[TShimmedFunc]
     session=None,  # type: Optional[Any]
+    verbosity=0,  # type: Optional[int]
 ):
     # (...) -> None
     """
@@ -735,6 +737,7 @@ def shim_unpack(
         to instantiate, if applicable.
     :param Optional[`~requests.Session`] session: A PipSession instance, defaults to
         None.
+    :param Optional[int] verbosity: 1 or 0 to indicate verbosity flag, defaults to 0.
     :return: The result of unpacking the url.
     :rtype: None
     """
@@ -769,6 +772,8 @@ def shim_unpack(
             assert session is not None
             assert progress_bar is not None
             unpack_kwargs[arg_name] = downloader_provider(session, progress_bar)
+        if "verbosity" in required_args:
+            unpack_kwargs["verbosity"] = verbosity
         return unpack_fn(**unpack_kwargs)  # type: ignore
 
 
@@ -812,10 +817,13 @@ def make_preparer(
     require_hashes=None,  # type: Optional[bool]
     use_user_site=None,  # type: Optional[bool]
     req_tracker=None,  # type: Optional[Union[TReqTracker, TShimmedFunc]]
+    build_tracker=None,  # type: Optional[Union[TBuildTracker, TShimmedFunc]]
     install_cmd_provider=None,  # type: Optional[TShimmedFunc]
     downloader_provider=None,  # type: Optional[TShimmedFunc]
     install_cmd=None,  # type: Optional[TCommandInstance]
     finder_provider=None,  # type: Optional[TShimmedFunc]
+    verbosity=0,  # type: Optional[int]
+    check_build_deps=False,  # type: Optional[bool]
 ):
     # (...) -> ContextManager
     """
@@ -851,6 +859,8 @@ def make_preparer(
         preparing requirements
     :param Optional[Union[TReqTracker, TShimmedFunc]] req_tracker: The requirement
         tracker to use for building packages, defaults to None
+    :param Optional[Union[TBuildTracker, TShimmedFunc]] build_tracker: The build
+        tracker to use for building packages, defaults to None and fallsback to req_tracker.
     :param Optional[TShimmedFunc] downloader_provider: A downloader provider
     :param Optional[TCommandInstance] install_cmd: The install command used to create
         the finder, session, and options if needed, defaults to None
@@ -883,7 +893,6 @@ def make_preparer(
         raise TypeError("No requirement tracker and no req tracker generator found!")
     if "downloader" in required_args and not downloader_provider:
         raise TypeError("no downloader provided, but one is required to continue!")
-    req_tracker_fn = resolve_possible_shim(req_tracker_fn)
     pip_options_created = options is None
     session_is_required = "session" in required_args
     finder_is_required = "finder" in required_args
@@ -924,11 +933,20 @@ def make_preparer(
         preparer_args["downloader"] = downloader_provider(session, progress_bar)
     if "in_tree_build" in required_args:
         preparer_args["in_tree_build"] = True
+    if "verbosity" in required_args:
+        preparer_args["verbosity"] = verbosity
+    if "check_build_deps" in required_args:
+        preparer_args["check_build_deps"] = check_build_deps
+    req_tracker_fn = resolve_possible_shim(req_tracker_fn)
     req_tracker_fn = nullcontext if not req_tracker_fn else req_tracker_fn
     with req_tracker_fn() as tracker_ctx:
         if "req_tracker" in required_args:
             req_tracker = tracker_ctx if req_tracker is None else req_tracker
             preparer_args["req_tracker"] = req_tracker
+        if "build_tracker" in required_args:
+            req_tracker = tracker_ctx if req_tracker is None else req_tracker
+            build_tracker = req_tracker if build_tracker is None else build_tracker
+            preparer_args["build_tracker"] = build_tracker
         preparer_args["lazy_wheel"] = True
         result = call_function_with_correct_args(preparer_fn, **preparer_args)
         yield result
@@ -1334,16 +1352,8 @@ def resolve(  # noqa:C901
             options=options,
             session=session,
             wheel_download_dir=wheel_download_dir,
-            **kwargs
+            **kwargs,
         )  # type: ignore
-        if getattr(reqset, "prepare_files", None):
-            reqset.add_requirement(ireq)
-            results = reqset.prepare_files(finder)
-            result = reqset.requirements
-            reqset.cleanup_files()
-            return result
-        if make_preparer_provider is None:
-            raise TypeError("Cannot create requirement preparer, cannot resolve!")
 
         preparer_args = {
             "build_dir": kwargs["build_dir"],
@@ -1380,18 +1390,32 @@ def resolve(  # noqa:C901
             options=options,
             install_cmd=install_command,
             wheel_cache=wheel_cache,
-            **resolver_args
+            **resolver_args,
         )  # type: ignore
         resolver.require_hashes = kwargs.get("require_hashes", False)  # type: ignore
         _, required_resolver_args = get_method_args(resolver.resolve)
         resolver_args = []
         if "requirement_set" in required_resolver_args.args:
-            reqset.add_requirement(ireq)
+            if hasattr(reqset, "add_requirement"):
+                reqset.add_requirement(ireq)
+            else:  # Pip >= 22.1.0
+                resolver._add_requirement_to_set(reqset, ireq)
             resolver_args.append(reqset)
         elif "root_reqs" in required_resolver_args.args:
             resolver_args.append([ireq])
         if "check_supported_wheels" in required_resolver_args.args:
             resolver_args.append(check_supported_wheels)
+        if getattr(reqset, "prepare_files", None):
+            if hasattr(reqset, "add_requirement"):
+                reqset.add_requirement(ireq)
+            else:  # Pip >= 22.1.0
+                resolver._add_requirement_to_set(reqset, ireq)
+            results = reqset.prepare_files(finder)
+            result = reqset.requirements
+            reqset.cleanup_files()
+            return result
+        if make_preparer_provider is None:
+            raise TypeError("Cannot create requirement preparer, cannot resolve!")
         result_reqset = resolver.resolve(*resolver_args)  # type: ignore
         if result_reqset is None:
             result_reqset = reqset
